"""Caracara Sensor Policies API Module."""

__all__ = [
    "SensorUpdatePoliciesApiModule",
]

from caracara.modules.sensor_update_policies.sensor_update_policies import (
    SensorUpdatePoliciesApiModule,
)
