"""Caracara PreventionPoliciesApiModule."""

__all__ = [
    "PreventionPoliciesApiModule",
]

from caracara.modules.prevention_policies.prevention_policies import (
    PreventionPoliciesApiModule,
)
